@@ -162,7 +162,7 @@
 #include <ctype.h>
 #include <string.h>
 
-SHP_CVSID("$Id: dbfopen.c 14869 2016-04-27 14:45:28Z strk $")
+SHP_CVSID("$Id: dbfopen.c 15722 2017-09-14 13:52:25Z pramsey $")
 
 #ifndef FALSE
 #  define FALSE		0
@@ -686,14 +686,22 @@ DBFCreateLL( const char * pszFilename, const char * pszCodePage, SAHooks *psHook
 /* -------------------------------------------------------------------- */
     fp = psHooks->FOpen( pszFullname, "wb" );
     if( fp == NULL )
+	{
+		free(pszBasename);
+		free(pszFullname);
         return( NULL );
+	}
 
     psHooks->FWrite( &chZero, 1, 1, fp );
     psHooks->FClose( fp );
 
     fp = psHooks->FOpen( pszFullname, "rb+" );
     if( fp == NULL )
+	{
+		free(pszBasename);
+		free(pszFullname);
         return( NULL );
+	}
 
 
     sprintf( pszFullname, "%s.cpg", pszBasename );
@@ -1727,13 +1735,13 @@ int SHPAPI_CALL DBFIsRecordDeleted( DBFHandle psDBF, int iShape )
 /* -------------------------------------------------------------------- */
 /*      Verify selection.                                               */
 /* -------------------------------------------------------------------- */
-    if( iShape < 0 || iShape >= psDBF->nRecords )
+    if( iShape < 0 || (psDBF->nRecords > 0 && iShape >= psDBF->nRecords) )
         return TRUE;
 
 /* -------------------------------------------------------------------- */
 /*	Have we read the record?					*/
 /* -------------------------------------------------------------------- */
-    if( !DBFLoadRecord( psDBF, iShape ) )
+    if( psDBF->nRecords > 0 && !DBFLoadRecord( psDBF, iShape ) )
         return FALSE;
 
 /* -------------------------------------------------------------------- */
