@@ -30,6 +30,11 @@
 #include "utils/jsonb.h"
 #endif
 
+#if POSTGIS_PGSQL_VERSION < 110
+/* See trac ticket #3867 */
+# define DatumGetJsonbP DatumGetJsonb
+#endif
+
 #include "uthash.h"
 
 #define FEATURES_CAPACITY_INITIAL 50
@@ -473,7 +478,6 @@ static void add_value_as_string(struct mvt_agg_context *ctx,
 static void parse_datum_as_string(struct mvt_agg_context *ctx, Oid typoid,
 	Datum datum, uint32_t *tags, uint32_t k)
 {
-	struct mvt_kv_string_value *kv;
 	Oid foutoid;
 	bool typisvarlena;
 	char *value;
@@ -587,7 +591,7 @@ static void parse_values(struct mvt_agg_context *ctx)
 		if (k == -1 && typoid != JSONBOID)
 			elog(ERROR, "parse_values: unexpectedly could not find parsed key name '%s'", key);
 		if (typoid == JSONBOID) {
-			tags = parse_jsonb(ctx, DatumGetJsonb(datum), tags);
+			tags = parse_jsonb(ctx, DatumGetJsonbP(datum), tags);
 			continue;
 		}
 #else
@@ -662,11 +666,9 @@ LWGEOM *mvt_geom(const LWGEOM *lwgeom, const GBOX *gbox, uint32_t extent, uint32
 	double width = gbox->xmax - gbox->xmin;
 	double height = gbox->ymax - gbox->ymin;
 	double resx = width / extent;
-	double resy = height / extent;
 	double fx = extent / width;
 	double fy = -(extent / height);
 	double buffer_map_xunits = resx * buffer;
-	double buffer_map_yunits = resy * buffer;
 	const GBOX *ggbox;
 	POSTGIS_DEBUG(2, "mvt_geom called");
 
@@ -807,7 +809,6 @@ void mvt_agg_transfn(struct mvt_agg_context *ctx)
 	LWGEOM *lwgeom;
 	VectorTile__Tile__Feature *feature;
 	VectorTile__Tile__Layer *layer = ctx->layer;
-	VectorTile__Tile__Feature **features = layer->features;
 	POSTGIS_DEBUG(2, "mvt_agg_transfn called");
 
 	if (layer->n_features >= ctx->features_capacity) {
